# coding: utf-8

from __future__ import unicode_literals

import time
import logging

from django.views import generic
from django import http
from django.views.decorators.csrf import csrf_exempt
from rest_framework import viewsets
from rest_framework.response import Response
from rest_framework import status
from rest_framework.decorators import detail_route


from . import models, utils, serializers

logger = logging.getLogger('jmb.webhooks')


class WebHookView(generic.View):

    @csrf_exempt
    def dispatch(self, request, *args, **kwargs):
        return super(WebHookView, self).dispatch(request, *args, **kwargs)

    def post(self, request):
        data = request.body
        t1 = time.time()
        n_events = utils.parse_webhook_data(data=data)
        t2 = time.time()
        msg = "Webhook: total registration time for %s events: %.3f"
        logger.info(msg % (n_events, t2 - t1))
        return http.HttpResponse(status=201)


class ContactViewSet(viewsets.ModelViewSet):

    """
    API endpoint that allows users to be viewed or edited.
    """
    queryset = models.Contact.objects.all().order_by('-date_create')
    serializer_class = serializers.ContactSerializer
    lookup_field = 'email'
    lookup_value_regex = '[-_0-9a-zA-Z.@]*'  # we want to include '@'

    @detail_route(methods=['get'])
    def opened_by_transmission(self, request, **filters):
        """Return the first date an email was opened. Used in
        invitation to know if sombody noticed the email"""

        obj = self.get_object()
        get_dict = {key: val for key, val in request.GET.items()}
        get_dict['opened__isnull'] = False
        last_email = obj.email_by_transmission(**get_dict).last()

        if last_email and last_email.opened:
            return Response({'date': last_email.opened}, status=status.HTTP_200_OK)
        return Response({}, status=status.HTTP_200_OK)


class RecipientListViewSet(viewsets.ModelViewSet):

    """
    API endpoint that allows groups to be viewed or edited.
    """
    queryset = models.RecipientList.objects.all()
    serializer_class = serializers.RecipientListSerializer


class GroupViewSet(viewsets.ModelViewSet):

    """
    API endpoint that allows groups to be viewed or edited.
    """
    queryset = models.Group.objects.all()
    serializer_class = serializers.GroupSerializer
    lookup_field = 'name'


class EventViewSet(viewsets.ModelViewSet):

    """
    API endpoint that allows groups to be viewed or edited.
    """
    queryset = models.Event.objects.all()
    serializer_class = serializers.EventSerializer


class TransmissionViewSet(viewsets.ModelViewSet):

    """
    API endpoint that allows groups to be viewed or edited.
    """
    queryset = models.Transmission.objects.all().order_by('-id')
    serializer_class = serializers.TransmissionSerializer

    @detail_route(methods=['post'])
    def send(self, request, *args, **kwargs):
        obj = self.get_object()
        obj.send()
        return Response(obj.get_data())


class TemplateViewSet(viewsets.ModelViewSet):

    """
    API endpoint that allows groups to be viewed or edited.
    """
    queryset = models.Template.objects.all()
    serializer_class = serializers.TemplateSerializer
    lookup_field = 'template_id'
